<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Settings - papert-lab</title>
    <link rel="icon" type="image/png" href="/static/favicon.png">
    <link href="/static\vendor\css\tailwind\tailwind.min.css" rel="stylesheet">
    <script src="/static\vendor\js\react\react.production.min.js"></script>
    <script src="/static\vendor\js\react\react-dom.production.min.js"></script>
    <script src="/static\vendor\js\babel\babel.min.js"></script>
</head>
<body class="bg-black text-green-400 font-mono">
    <div id="root"></div>
    <script type="text/babel" src="/static/settings.js"></script>
</body>
</html>
